// Package model provides repository models and persistent interface for the service
package model

import (
	"context"

	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"

	"github.com/cenkalti/backoff/v4"
	"gitlab.mashplace.com/mashplace/backend/common/api/generic/query"
	"gitlab.mashplace.com/mashplace/backend/common/generic/dbfilter"
	"gitlab.mashplace.com/mashplace/backend/common/generic/migrations"
)

// Repository for persistent operations
type Repository interface {
	MigrateSchema(ctx context.Context, ms ...migrations.Migration) error
	MigrateData(
		ctx context.Context,
		backoff backoff.BackOff,
		ms ...*migrations.DataMigration,
	)
	InvitationSave(ctx context.Context, invitation *Invitation) error
	InvitationGetByID(ctx context.Context, id string) (*Invitation, error)
	InvitationGetByFilters(ctx context.Context, fs ...dbfilter.Condition) (*Invitation, error)
	InvitationListFilter(ctx context.Context, fs *query.QueryParams) ([]*Invitation, int64, error)
	InvitationListByFilters(ctx context.Context, fs ...dbfilter.Condition) ([]*Invitation, error)
	InvitationUpdateNotified(ctx context.Context, id, notification string) error
	InvitationUpdateStatus(ctx context.Context, id string, status invitation.InvitationStatus) error
	InvitationUpdateInviteeID(ctx context.Context, email, userID string) ([]*Invitation, error)
	ContactValidationSave(ctx context.Context, cv *ContactValidation) error
	ContactValidationRemoveByID(ctx context.Context, id string) error
	ContactValidationGetLatestByParams(
		ctx context.Context,
		typ invitation.ContactValidationType,
		userID, key, remote string,
	) (*ContactValidation, error)
	ContactValidationGetByFilters(ctx context.Context, fs ...dbfilter.Condition) (*ContactValidation, error)
}
