package model

import (
	"time"

	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

// User model
type User struct {
	UserID string `bson:"user_id,omitempty"`
	Email  string `bson:"email,omitempty"`
	Name   string `bson:"name,omitempty"`
	Phone  string `bson:"phone,omitempty"`
}

// Property model
type Property struct {
	PropertyID string `bson:"property_id,omitempty"`
	VersionID  string `bson:"version_id,omitempty"`
}

// Invitation model
type Invitation struct {
	ID            primitive.ObjectID          `bson:"_id,omitempty"`
	Type          invitation.InvitationType   `bson:"type"`
	HostID        string                      `bson:"host_id"`
	Invitee       User                        `bson:"invitee"`
	Otp           string                      `bson:"otp"`
	Property      Property                    `bson:"property"`
	Status        invitation.InvitationStatus `bson:"status"`
	CreatedAt     time.Time                   `bson:"created_at"`
	ExpiresAt     time.Time                   `bson:"expires_at"`
	Offer         string                      `bson:"offer"`
	NotifyURL     string                      `bson:"notify_url"`
	Message       string                      `bson:"message"`
	Notifications NotificationSentFlags       `bson:"notifications"`
	UpdatedAt     time.Time                   `bson:"updated_at,omitempty"`
}

// NotificationSentFlags model
type NotificationSentFlags struct {
	LandlordPendingOver24H time.Time `bson:"landlord_pending_over_24h,omitempty"`
	Expire                 time.Time `bson:"expire,omitempty"`
}
