// Package s1v0001 schema migration
package s1v0001

import (
	"context"
	"time"

	"go.mongodb.org/mongo-driver/bson"

	"go.mongodb.org/mongo-driver/mongo"
)

// Up migration
func Up(ctx context.Context, db *mongo.Database, migrationID string) (affected int, err error) {
	inv := db.Collection("invitations")

	_, err = inv.UpdateMany(ctx, bson.M{}, bson.M{
		"$rename": bson.M{
			"inviter_id":   "host_id",
			"callback_url": "notify_url",
		},
	})
	if err != nil {
		return -1, err
	}

	_, err = inv.UpdateMany(ctx, bson.M{}, bson.A{
		bson.M{
			"$set": bson.M{
				"invitee": bson.M{
					"user_id": "$invitee_id",
					"name":    bson.M{"$concat": bson.A{"$invite_first_name", " ", "$invite_last_name"}},
					"email":   "$invitee_email",
					"phone":   "$invitee_phone",
				},
				"property": bson.M{
					"property_id": "$property_id",
					"version_id":  "",
				},
			},
		},
	})
	if err != nil {
		return -1, err
	}

	_, err = inv.UpdateMany(ctx, bson.M{}, bson.A{
		bson.M{
			"$set": bson.M{
				"process": bson.M{"$toString": "$process"},
			},
		},
	})
	if err != nil {
		return -1, err
	}

	_, err = inv.UpdateMany(ctx, bson.M{"notifcation.landlord_pending_over_24h": true}, bson.M{
		"$set": bson.M{
			"notifcation.landlord_pending_over_24h": time.Now(),
		},
	})
	if err != nil {
		return -1, err
	}

	_, err = inv.UpdateMany(ctx, bson.M{"notifcation.landlord_pending_over_24h": bson.M{"$ne": true}}, bson.M{
		"$unset": bson.M{
			"notifcation.landlord_pending_over_24h": "",
		},
	})
	if err != nil {
		return -1, err
	}

	cols, err := db.ListCollectionNames(ctx, bson.M{"name": "contact-validations"})
	if err != nil {
		return -1, err
	}

	if len(cols) > 0 {
		err = db.Client().Database("admin").RunCommand(ctx, bson.D{
			{Key: "renameCollection", Value: db.Name() + ".contact-validations"},
			{Key: "to", Value: db.Name() + ".contact_validations"},
		}).Err()
		if err != nil {
			return -1, err
		}

		cv := db.Collection("contact_validations")

		_, err = cv.UpdateMany(ctx, bson.M{}, bson.M{
			"$rename": bson.M{
				"account_id": "user_id",
			},
		})
		if err != nil {
			return -1, err
		}
	}

	return -1, nil
}
