package service

import (
	"context"
	"encoding/json"
	"net/http"
	"strings"

	"gitlab.mashplace.com/mashplace/backend/common/api/grpcerr"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"
	messaging "gitlab.mashplace.com/mashplace/backend/messaging/api"
	notification "gitlab.mashplace.com/mashplace/backend/notification/api"
)

func (srv *serviceImpl) LandlordReferenceSave(
	ctx context.Context,
	req *invitation.LandlordReferenceSaveRequest,
) (*invitation.LandlordReferenceSaveResponse, error) {
	switch {
	case req.GetInvitee().GetName() == "",
		req.GetInvitee().GetPhone() == "",
		req.GetInvitee().GetEmail() == "",
		req.GetHost().GetName() == "",
		req.GetHost().GetPhone() == "",
		req.GetHost().GetEmail() == "":
		return nil, grpcerr.InvalidData
	}

	u := srv.Config.ReferenceSheetURL

	qr := u.Query()

	qr.Set("invitee_email", req.GetInvitee().GetEmail())
	qr.Set("invitee_name", req.GetInvitee().GetName())
	qr.Set("invitee_phone", req.GetInvitee().GetPhone())
	qr.Set("inviter_email", req.GetHost().GetEmail())
	qr.Set("inviter_name", req.GetHost().GetName())
	qr.Set("inviter_phone", req.GetHost().GetPhone())

	u.RawQuery = qr.Encode()

	getResp, err := http.Get(u.String())
	if err != nil {
		return nil, err
	}

	defer func() {
		_ = getResp.Body.Close()
	}()

	if getResp.StatusCode != http.StatusOK {
		return nil, grpcerr.ProviderError
	}

	bs, _ := json.Marshal(map[string]interface{}{
		"user": map[string]interface{}{
			"name":  req.GetHost().GetName(),
			"email": req.GetHost().GetEmail(),
			"phone": req.GetHost().GetPhone(),
		},
	})

	_, err = srv.Connections.Notification.SendNotification(ctx, &notification.SendNotificationRequest{
		Entity: notification.Entity_ENTITY_PROPERTY_INVITATION_REFER_LANDLORD,
		Action: notification.Action_ACTION_CREATE,
		Targets: []*notification.Target{
			{
				Role: notification.Role_ROLE_LANDLORD,
				Contacts: &notification.Contacts{
					Email:    strings.ToLower(req.GetInvitee().GetEmail()),
					Phone:    req.GetInvitee().GetPhone(),
					Fullname: req.GetInvitee().GetName(),
				},
			},
		},
		Embeds: []*notification.EventEmbed{
			{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_JSON,
				EmbedValue: string(bs),
			},
		},
	})
	if err != nil {
		log.WithError(err).Error("sending refer landlord notification error")
		return nil, err
	}

	return &invitation.LandlordReferenceSaveResponse{}, nil
}
