// Package expire times out pending invitations
package expire

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/api/grpcctx"
	"gitlab.mashplace.com/mashplace/backend/common/generic/dbfilter"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/notifications"
	notification "gitlab.mashplace.com/mashplace/backend/notification/api"
	schedule "gitlab.mashplace.com/mashplace/backend/schedule/api"
)

// Config for a task
type Config struct {
	Repository   model.Repository
	Notification notification.NotificationServiceClient
}

// New returns new task instance
func New(config *Config) (schedule.Handler, error) {
	return &handlerImpl{
		Config: *config,
	}, nil
}

type handlerImpl struct {
	Config
}

func (h *handlerImpl) Name() string {
	return "invitation-expire"
}

func (h *handlerImpl) Consume(
	ctx context.Context,
	req *schedule.Reminder_ConsumeRequest,
	data []byte,
	now time.Time,
) error {
	bgctx := grpcctx.ValuesBackgroundContext(ctx)

	go h.process(bgctx, now)

	return nil
}

func (h *handlerImpl) process(ctx context.Context, now time.Time) {
	models, err := h.Repository.InvitationListByFilters(
		ctx,
		dbfilter.Eq("status", invitation.InvitationStatus_INVITATION_STATUS_PENDING),
		dbfilter.Lt("expire_at", now),
		dbfilter.Not(dbfilter.Gt("notifications.expire", time.Time{})),
	)
	if err != nil {
		return
	}

	for _, m := range models {
		m.Status = invitation.InvitationStatus_INVITATION_STATUS_TIMEOUT
		m.Notifications.Expire = now

		err = h.Repository.InvitationSave(ctx, m)
		if err != nil {
			log.WithError(err).Error("updating invitation status to timeout: " + m.ID.Hex())

			continue
		}

		notifications.HandleInvitationStatusChange(ctx, h.Notification, m)
	}
}
